/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/** @file
 * @defgroup tw_sensor_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Sensor Example main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 */

#include <stdbool.h>
#include <string.h>
#include <stdint.h>
#include <stdio.h>
#include "app_util_platform.h"
#include "app_error.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "app_uart.h"

#define NRF_LOG_MODULE_NAME "APP"
#include "nrf.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

/* TWI PINS */
#define TWI_SCL_PIN     22
#define TWI_SDA_PIN     20

/* MPU-6050 I2C-ADDRESS */
#define MPU_6050_I2C_ADDR   0x69
//#define MPU_6050_I2C_ADDR   0xD0

/* MPU-6050 Register Map */
#define WHO_AM_I            0x75
#define ACCEL_XOUT_H        0x3B
#define TEMP_OUT_H          0x41
#define GYRO_XOUT_H         0x43
#define PWR_MGMT_1          0x6B

#define TX_PIN_NUMBER  13
#define RX_PIN_NUMBER  15
#define RTS_PIN_NUMBER 12
#define CTS_PIN_NUMBER 14

//#define ENABLE_LOOPBACK_TEST  /**< if defined, then this example will be a loopback test, which means that TX should be connected to RX to get data loopback. */

#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 256                         /**< UART RX buffer size. */

/* TWI instance ID. */
#define TWI_INSTANCE_ID     0

/* Common addresses definition for temperature sensor. */
#define LM75B_ADDR          (0x90U >> 1)

#define LM75B_REG_TEMP      0x00U
#define LM75B_REG_CONF      0x01U
#define LM75B_REG_THYST     0x02U
#define LM75B_REG_TOS       0x03U

/* Mode for LM75B. */
#define NORMAL_MODE 0U

/* Indicates if operation on TWI has ended. */
static volatile bool m_xfer_done = false;

/* TWI instance. */
static const nrf_drv_twi_t m_twi_mpu_6050 = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);

/* Buffer for samples read from temperature sensor. */
static uint8_t m_sample;

uint8_t device_id=0x00;

void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
}

/**
 * @brief Function for handling data from temperature sensor.
 *
 * @param[in] temp          Temperature in Celsius degrees read from sensor.
 */
__STATIC_INLINE void data_handler(uint8_t temp)
{
    NRF_LOG_INFO("Temperature: %d Celsius degrees.\r\n", temp);
}

/**
 * @brief TWI events handler.
 */
void twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{
    switch (p_event->type)
    {
        case NRF_DRV_TWI_EVT_DONE:
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_RX)
            {
                data_handler(m_sample);
            }
            m_xfer_done = true;
            break;
        default:
            break;
    }
}

/**
 * @brief UART initialization.
 */
void twi_init (void)
{
	 ret_code_t err_code;

	    const nrf_drv_twi_config_t twi_mpu_6050_config = {
	        .scl            = TWI_SCL_PIN,
	        .sda            = TWI_SDA_PIN,
	        .frequency      = NRF_TWI_FREQ_400K,
	        .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
			.clear_bus_init     = false
	    };
	  err_code = nrf_drv_twi_init(&m_twi_mpu_6050, &twi_mpu_6050_config, twi_handler, NULL);
	    switch(err_code)
	    {
	    case NRF_SUCCESS:printf("\r\n NRF_SUCCESS in: \r\n"); //If the procedure was successful.
	    	break;
	    case NRF_ERROR_BUSY:printf("\r\n NRF_ERROR_BUSY in: \r\n"); //If some other peripheral with the same instance ID is already in use. This is possible only if PERIPHERAL_RESOURCE_SHARING_ENABLED is set to a value other than zero.
	      	break;
	    case NRF_ERROR_INVALID_STATE: printf("\r\n NRF_ERROR_INVALID_STATE in: \r\n");  //If the driver is in invalid state.
	      	break;

	    }
	    APP_ERROR_CHECK(err_code);

	    nrf_drv_twi_enable(&m_twi_mpu_6050);
}
ret_code_t mpu_6050_I2C_register_write(uint8_t reg_addr, uint8_t * p_tx_data, uint8_t bytes)
{

    ret_code_t ret_code;

    uint8_t tx_data[bytes+1];

    tx_data[0] = reg_addr;

    for(uint8_t i = 0 ; i<bytes ; i++)
    {
        tx_data[i+1] = p_tx_data[i];
    }
    ret_code = nrf_drv_twi_tx(&m_twi_mpu_6050, MPU_6050_I2C_ADDR, tx_data, sizeof(tx_data), false);


    switch(ret_code)
    {
    case NRF_SUCCESS:printf("\r\n NRF_SUCCESS write tx: \r\n"); //If the procedure was successful.
    	break;
    case NRF_ERROR_BUSY:printf("\r\n NRF_ERROR_BUSY write tx: \r\n"); //If the driver is not ready for a new transfer.
      	break;
    case NRF_ERROR_INTERNAL: printf("\r\n NRF_ERROR_INTERNAL write tx: \r\n");  //If an error was detected by hardware.
      	break;
    case NRF_ERROR_INVALID_ADDR:printf("\r\n NRF_ERROR_INVALID_ADDR write tx: \r\n");// If the EasyDMA is used and memory adress in not in RAM.
      	break;
    case NRF_ERROR_DRV_TWI_ERR_ANACK:printf("\r\n NRF_ERROR_DRV_TWI_ERR_ANACK write tx: \r\n");// If NACK received after sending the address.
      	break;
    case NRF_ERROR_DRV_TWI_ERR_DNACK:printf("\r\n NRF_ERROR_DRV_TWI_ERR_DNACK write tx: \r\n");// If NACK received after sending a data byte.
      	break;
    }
    return ret_code;
}

/**
 * @brief Function for reading from registers on the MPU-6050.
 */

 ret_code_t mpu_6050_I2C_register_read( uint8_t reg_addr,  uint8_t * p_rx_data, uint32_t bytes)
{
    ret_code_t ret_code;

    ret_code = nrf_drv_twi_tx(&m_twi_mpu_6050,MPU_6050_I2C_ADDR, &reg_addr, 1, false);

    if(ret_code != NRF_SUCCESS)
    {
        return ret_code;
    }
    switch(ret_code)
    {
    case NRF_SUCCESS:printf("\r\n NRF_SUCCESS read Tx: \r\n"); //If the procedure was successful.
    	break;
    case NRF_ERROR_BUSY:printf("\r\n NRF_ERROR_BUSY read Tx: \r\n"); //If the driver is not ready for a new transfer.
      	break;
    case NRF_ERROR_INTERNAL: printf("\r\n NRF_ERROR_INTERNAL read Tx: \r\n");  //If an error was detected by hardware.
      	break;
    case NRF_ERROR_INVALID_ADDR:printf("\r\n NRF_ERROR_INVALID_ADDR read Tx: \r\n");// If the EasyDMA is used and memory adress in not in RAM.
      	break;
    case NRF_ERROR_DRV_TWI_ERR_ANACK:printf("\r\n NRF_ERROR_DRV_TWI_ERR_ANACK read Tx: \r\n");// If NACK received after sending the address.
      	break;
    case NRF_ERROR_DRV_TWI_ERR_DNACK:printf("\r\n NRF_ERROR_DRV_TWI_ERR_DNACK read Tx: \r\n");// If NACK received after sending a data byte.
      	break;
    }
    while(!m_xfer_done);
    //m_xfer_done = false;
    ret_code = nrf_drv_twi_rx(&m_twi_mpu_6050, MPU_6050_I2C_ADDR, p_rx_data, bytes);
    switch(ret_code)
      {
      case NRF_SUCCESS:printf("\r\n NRF_SUCCESS read Rx: \r\n"); //If the procedure was successful.
      	break;
      case NRF_ERROR_BUSY:printf("\r\n NRF_ERROR_BUSY read Rx: \r\n"); //If the driver is not ready for a new transfer.
        	break;
      case NRF_ERROR_INTERNAL: printf("\r\n NRF_ERROR_INTERNAL read Rx: \r\n");  //If an error was detected by hardware.
        	break;
      case NRF_ERROR_INVALID_ADDR:printf("\r\n NRF_ERROR_INVALID_ADDR read Rx: \r\n");// If the EasyDMA is used and memory adress in not in RAM.
        	break;
      case NRF_ERROR_DRV_TWI_ERR_ANACK:printf("\r\n NRF_ERROR_DRV_TWI_ERR_ANACK read Rx: \r\n");// If NACK received after sending the address.
        	break;
      case NRF_ERROR_DRV_TWI_ERR_DNACK:printf("\r\n NRF_ERROR_DRV_TWI_ERR_DNACK rea Rx: \r\n");// If NACK received after sending a data byte.
        	break;
      }
    return ret_code;
}

/*
void mpu_6050_init (void)
{
    ret_code_t err_code;

    uint8_t tx_data = 0x00;

    // Write zero to the PWR_MGMT_1 register to wake up the MPU-6050
    err_code = mpu_6050_I2C_register_write(PWR_MGMT_1, &tx_data, 1);

    APP_ERROR_CHECK(err_code);
}*/
/*
void mpu_6050_get_device_id (uint8_t * p_dev_id)
{
    ret_code_t err_code;

    uint8_t rx_data;

    // Read the I2C Address of the MPU-6050 from the WHO_AM_I register
    err_code = mpu_6050_I2C_register_read(WHO_AM_I,&rx_data,1);

    APP_ERROR_CHECK(err_code);

    *p_dev_id = rx_data;
}*/
/**
 * @brief Function for main application entry.
 */
int main(void)
{
	  uint32_t err_code;
	  //  bsp_board_leds_init();
	    const app_uart_comm_params_t comm_params =
	      {
	          RX_PIN_NUMBER,
	          TX_PIN_NUMBER,
	          RTS_PIN_NUMBER,
	          CTS_PIN_NUMBER,
	          APP_UART_FLOW_CONTROL_ENABLED,
	          false,
	          UART_BAUDRATE_BAUDRATE_Baud115200
	      };

	    APP_UART_FIFO_INIT(&comm_params,
	                         UART_RX_BUF_SIZE,
	                         UART_TX_BUF_SIZE,
	                         uart_error_handle,
	                         APP_IRQ_PRIORITY_LOWEST,
	                         err_code);


	    APP_ERROR_CHECK(err_code);
	printf("\r\nStart1: \r\n");
    twi_init();
    printf("\r\nStart2: \r\n");
    nrf_delay_ms(100);
    uint8_t tx_data = 0x00;

       // Write zero to the PWR_MGMT_1 register to wake up the MPU-6050
       mpu_6050_I2C_register_write(PWR_MGMT_1, &tx_data, 1);
    // mpu_6050_init();
    printf("\r\nStart3: \r\n");
    nrf_delay_ms(100);
    //nrf_drv_twi_disable(&m_twi_mpu_6050);
    //nrf_drv_twi_enable(&m_twi_mpu_6050);

    mpu_6050_I2C_register_read( WHO_AM_I,&device_id, 1);

    app_uart_put(device_id);

    while (true)
    {
    	//printf("\r\nStart3: \r\n");
    	//nrf_delay_ms(100);

    }
}

/** @} */
